package de.be4.smvclipse;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PushbackReader;

import de.be4.smvclipse.prolog.AST2PrologTransformator;
import de.be4.smvclipse.prolog.exception.TooFewArgumentsException;
import de.be4.smvclipse.sablecc.lexer.Lexer;
import de.be4.smvclipse.sablecc.lexer.LexerException;
import de.be4.smvclipse.sablecc.node.Start;
import de.be4.smvclipse.sablecc.node.Switch;
import de.be4.smvclipse.sablecc.parser.Parser;
import de.be4.smvclipse.sablecc.parser.ParserException;

/**
 * @author matthaeus
 *
 * This class is suppossed to invoke the Parser and apply the Prolog transformation to the AST.
 * The result is representation of the SMV programme as a Prolog term.
 */
public class Starter {

	public static void main(String[] args) {
		//declaration 
		Parser pi = null;
		Start tree = null;
		
		try {
			//create parser instance 
			if ( args.length < 2 ) {
				throw new TooFewArgumentsException();
			} else if ( args[0].equals("System.in") ) {
				pi = new Parser(new Lexer(new PushbackReader(new InputStreamReader(System.in))));
			} else if ( !new File(args[0]).exists() ) {
				throw new FileNotFoundException("The input file for the parser could not be found.");
			} else {
				pi = new Parser(new Lexer(new PushbackReader(new InputStreamReader(new FileInputStream(args[0])))));
			}			

			tree = pi.parse();
		} catch (ParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LexerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TooFewArgumentsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(0);
		}

		apply(tree, new AST2PrologTransformator(args[1]));
//		System.out.println("\nASTPrinter Output:\n");
//		apply(tree, new ASTPrinter());
		
		//debugging
//		ASTPrinter dis = new ASTPrinter();
//		tree.apply(dis);
	}
	
	private static void apply(Start tree, Switch switchh) {
		tree.apply(switchh);
	}
}
